/* Project header file */

#ifndef ADOManagerQueryH
#define ADOManagerQueryH
//---------------------------------------------------------------------------
#include <ADODB.hpp>
#include <DB.hpp>
#include <TntClasses.hpp>
#include <Lmcons.h>
#include "LoginForm.h"
//---------------------------------------------------------------------------
struct TProcessInfo
{
	AnsiString ID;
	WideString LoginName, DBName, DBUser, ServerName;
	__fastcall TProcessInfo();
	__fastcall ~TProcessInfo();
	void __fastcall Reset();
};
//---------------------------------------------------------------------------
struct TLoginInfo
{
	/* Hold Login name, Windows account name, or Windows group name,
	   used in various functions                                     */
	WideString LoginName;
	WideString Password;
	/* Login's old password, used when changing login password. */
	WideString OldPassword;
	WideString DefaultDatabase;
	WideString Role; /* This is Database role, Server role, or Application role */
	WideString UserName; /* Database User name, used in various functions */
	bool Option;
	void __fastcall Reset();
};
//---------------------------------------------------------------------------
struct TDBBackupInfo
{
private:
	/* Database file list used to restore or backup database */
	TTntStringList *DBFileList;
	/* List of new path name for database's files use with
	   RestoreDBFiles                                      */
	TTntStringList *NewPathList;
public:
	/* Name of Database to backup or restore */
	WideString DBName;
	/* Backup device name */
	WideString DeviceName;
	 /* Device is a Disk file or not. Backup or restore from disk
	    \file                                                     */	
	bool DiskFile;
	/* Backup set number used to backup or restore database */
	int BackupSetNumber;
	/* Path to database log file name use for restore database log */
	WideString LogFileName;
	/* New path to file name use for restore database log */
	WideString NewLogPath;
	__fastcall TDBBackupInfo();
	__fastcall ~TDBBackupInfo();
	/* Add file to DBFileList member */
	void __fastcall AddBackupFile(WideString FileName);
	void __fastcall DeleteBackupFile(WideString FileName);
	/* Add file to NewPathList member */
	void __fastcall AddNewFilePath(WideString FileName);
	void __fastcall DeleteBackupFilePath(WideString FileName);
	void __fastcall Reset();
	void __fastcall ResetDBFileList();
	void __fastcall ResetPathList();
	/* Number of Database files in DBFileList member */
	int __fastcall DBFileCount();
	int __fastcall FilePathCount();
	WideString __fastcall GetNewFilePath(int index);
	WideString __fastcall GetDBFile(int index);
};
/* Structure used with SetPermissions */
struct TPermissionInfo
{
	/* User name or group */
	WideString UserOrGroup;
	/* Object name */
	WideString Object;
	/* INSERT, DELETE, UPDATE ... */
	AnsiString Permission;
	/* If true, deny permissions */
	bool Deny;
};
//---------------------------------------------------------------------------
class PACKAGE TADOSqlManagerQuery : public TADOQuery
{
private:
	TProcessInfo* FCurrentProcessInfo;
	bool FDisplayError;
	AnsiString FComputerName;
	AnsiString FUserName;
protected:
	AnsiString FLastError;
	WideString __fastcall ReadDatabaseName();
	void __fastcall SetDatabase(WideString Databasename);
	WideString __fastcall ReadServerName();
	WideString __fastcall ReadLoginName();
	WideString __fastcall ReadUserName();
	AnsiString __fastcall ReadID();
public:
	__fastcall TADOSqlManagerQuery(TComponent* Owner);
	__fastcall ~TADOSqlManagerQuery();
	void __fastcall LoginFormDestroy();
	bool __fastcall UpdateProcessInfo();
	/* Report SQL server properties */
	bool __fastcall ServerInfo(TTntStringList* VersionInfo);
	/* Return SQL Server version information. In addition to
	   \version information regarding the actual build number of the
	   server, various environment information is also returned      */
	bool __fastcall GetServerOS(TTntStringList *ServerOS);
	/* Shut down SQL server */
	bool __fastcall Shutdown(bool NoWait);
	int __fastcall LoginPrompt();
	/* Provides information about current Microsoft SQL Server
	   users and processes.
	   
	   Connections: Connection list
	   
	   UserOnly: List only user connections
	   
	   After return, Connections will contain XML data in the
	   following format:
	     * \<Connections\>
	       * \<Connection\>
	         * \<ProcessID\>\</ProcessID\>
	         * \<ContextID\>\</ContextID\>
	         * \<Status\>\</Status\>
	         * \<Login\>\</Login\>
	         * \<Hostname\>\</Hostname\>
	         * \<BlockingProcess\>\</BlockingProcess\>
	         * \<Database\>\</Database\>
	         * \<Command\>\</Command\>
	       * \</Connection\>
	     * \</Connections\>                                      */
	bool __fastcall GetConnections(TTntStringList* Connections, bool UserOnly);
	/* Terminate connections */
	bool __fastcall TerminateConnections(TStringList* PIDList);
	/* Disconnect */
	void __fastcall Disconnect();
	bool __fastcall GetLogins(TTntStringList* LoginList);
	/* Provides information about login and the associated users in
	   each database.
	   
	   LoginProperties: Login property list
	   
	   LoginName: Login name
	   
	   DBAccess: If true, report users and aliases associated with
	   login name in each database
	   
	   After return, LoginProperties will contain XML data in the
	   following format:
	     * \<Login\>
	       * \<Name\>
	       * \<DefaultDatabase\>
	       * \<DefLangName\>\</DefLangName\>
	       * \<IsUser\>\</IsUser\>
	       * \<IsRemote\>\</IsRemote\>
	       * \<Databases\>
	         * \<DATABASENAME\>
	           * \<DBRole\>\</DBRole\> or
	           * \<UserName\>\</UserName\>
	         * \</DATABASENAME\>
	       * \</Databases\>
	     * \</Login\>                                               */
	bool __fastcall GetLoginProperties(TTntStringList* LoginProperties, WideString LoginName, bool DBAccess);
	/* Creates new Microsoft SQL Server logins that allows a user
	   to connect to an instance of SQL Server using SQL Server
	   Authentication.
	   
	   LoginInfos: Pointer to array of TLoginInfo
	   
	   Count: Number of array elements
	   
	   This function uses LoginName, Password, DefaulDatabase, Role
	   \members of TLoginInfo
	   
	   LoginName: SQL Server's login account name
	   
	   Password: Password
	   
	   DefaultDatabase: Default database for the login
	   
	   Role: Server role of the new login                           */
	bool __fastcall CreateLogins(TLoginInfo* LoginInfos, int Count);
	/* Return encrypted password
	   
	   Name: Login name          */
	char* __fastcall GetLoginPassword(WideString Name);
	/* Change logins passwords
	   
	   LoginInfos: Pointer to array of TLoginInfo
	   
	   Count: Number of array element
	   
	   This function uses OldPassword, Password, and LoginName
	   \members of TLoginInfo Structure                        */
	bool __fastcall ChangePasswords(TLoginInfo* LoginInfos, int Count);
	/* Removes Microsoft SQL Server logins, preventing access to
	   SQL Server using those login names
	   
	   LoginInfos: Pointer to array of TLoginInfo
	   
	   Count: Number of array element
	   
	   This function uses LoginName member of TLoginInfo
	   
	   LoginName: Login account to be removed                      */
	bool __fastcall DeleteLogins(TLoginInfo* LoginInfos, int Count);
	/* Get all login's encrypted passwords
	   
	   After return, Logins will contain XML data in the following
	   format
	     * <c>\<Logins\></c>
	       * <c>\<Login\></c>
	         * <c>\<Name\>sa\</Name\></c>
	         * <c>\<Paswword\>PASSWORD\</Password\></c>
	       * <c>\</Login\></c>
	     * <c>\</Logins\></c>                                      */
	bool __fastcall GetAllLoginPassword(TTntStringList* Logins);
	bool __fastcall CreateUsers(TLoginInfo* LoginInfos, int Count);
	/* Changes the relationship between a Microsoft SQL Server
	   login and a SQL Server user in the current database.
	   
	   LoginInfos: Pointer to array of TLoginInfo
	   
	   Count: Number of array elements
	   
	   This function uses UserName, LoginName members of TLoginInfo
	   
	   UserName: Database user name
	   
	   LoginName: SQL Server's login account name                   */
	bool __fastcall ChangeUsersLogins(TLoginInfo* LoginInfos, int Count);
	/* \Reports information about Microsoft SQL Server users,
	   Microsoft Windows NT users, and database roles in the
	   current database.                                        */
	bool __fastcall GetUsers(TTntStringList* UserList);
	/* Removes security accounts from the current database.
	   
	   LoginInfos: Pointer to array of TLoginInfo
	   
	   Count: Number of array element
	   
	   This function uses UserName member of TLoginInfo
	   
	   UserName: Is the name of the account to be removed. UserName can
	   be the name of a Microsoft SQL Server user, or Microsoft
	   Windows NT user or group, and must exist in the current
	   database. When specifying a Windows NT user or group, specify
	   the name the Windows NT user or group is known by in the
	   database                                                         */
	bool __fastcall DeleteUsers(TLoginInfo* LoginInfos, int Count);
	bool __fastcall HasAccessDatabases(TTntStringList* DatabaseList);
	/* Lists databases that reside in an instance of Microsoft SQL
	   Server or are accessible through a database gateway
	   
	   After return DatabaseList will contains XML data in the
	   following format:
	     * \<Databases\>
	       * \<Database\>
	         * \<Name\>\</Name\>
	         * \<Size\>\</Size\>
	       * \</Database\>
	     * \</Databases\>                                           */
	bool __fastcall ListDatabases(TTntStringList* DatabaseList);
	/* \Reports information about all databases.
	   
	   DatabaseList: Pointer to a TTntStringList
	   
	   Status: If true, report databases' status
	   
	   After return, DatabaseList will contain XML data in the
	   following format
	     * <c>\<Databases\></c>
	       * <c>\<Database\></c>
	         * <c>\<Name\>master\</Name\></c>
	         * <c>\<Size\>4.50 MB\</Size\></c>
	         * <c>\<Owner\>sa\</Owner\></c>
	         * <c>\<DateCreated\>Apr 8 2003\</DateCreated\></c>
	         * <c>\<Level\>90\</Level\></c>
	         * <c>\<Status\>\</Status\></c>
	       * <c>\</Database\></c>
	     * <c>\</Databases\></c>                                */
	bool __fastcall GetAllDatabases(TTntStringList* DatabaseList, bool Status);
	/* Create one or more databases
	   
	   DatabaseName: List of databases to create */
	bool __fastcall CreateDatabases(TTntStringList *DatabaseNames);
	/* Removes one or more databases from Microsoft SQL Server.
	   Removing a database deletes the database and the disk files
	   used by the database.
	   
	   DatabaseName: Database name list                            */
	bool __fastcall DeleteDatabases(TTntStringList *DatabaseNames);
	/* \Reports information about a specified database.
	   
	   DBProperties: Database property list
	   
	   DBName: Database name
	   
	   DBFiles: If true, report database's files
	   
	   After return, DBProperties will contain XML data in the
	   following format:
	     * \<DBProperties\>
	       * \<Name\>\</Name\>
	       * \<Size\>\</Size\>
	       * \<Owner\>\</Owner\>
	       * \<DateCreated\>\</DateCreated\>
	       * \<Level\>\</Level\>
	       * \<Status\>\</Status\>
	       * \<DBFiles\>
	         * \<File\>
	         * \<Name\>\</Name\>
	         * \<FileGroup\>\</FileGroup\>
	         * \<Size\>\</Size\>
	         * \<Path\>\</Path\>
	         * \<MaxSize\>\</MaxSize\>
	         * \<Growth\>\</Growth\>
	         * \<Usage\>\</Usage\>
	         * \</File\>
	       * \</DBFiles\>
	     * \</DBProperties\>                                   */
	bool __fastcall GetDBProperties(TTntStringList* DBProperties, WideString DBName, bool DBFiles);
	/* Detaches a database from a server and, optionally, runs
	   UPDATE STATISTICS on all tables before detaching
	   
	   DBName: Database name
	   
	   SkipCheck: If true, UPDATE STATISTICS is skipped. If false,
	   UPDATE STATISTICS is run. This option is useful for databases
	   that are to be moved to read-only media.
	                                                                 */
	bool __fastcall DetachDB(WideString DBName, bool SkipCheck);
	/* Attaches a database to a server
	   
	   DBName: Database name
	   
	   FileList: Database files        */
	bool __fastcall AttachDB(WideString DBName, TTntStringList* FileList);
	/* Attaches a database having only one data file to the current
	   server.
	   
	   DBName: Database name
	   
	   FileName: Path to database file                              */
	bool __fastcall AttachDBSingleFile(WideString DBName, WideString FileName);
	/* Changes the owner of the current database
	   
	   NewOwner: New owner
	   
	   RemapAlias: Is the value true or false, which indicates
	   whether existing aliases to the old database owner (dbo) are
	   mapped to the new owner of the current database or dropped   */
	bool __fastcall ChangeDBOwner(WideString NewOwner, bool RemapAlias);
	/* Shrinks the size of the data files in the specified databases
	   
	   DBNames: Database names
	   
	   Percents: Pointer to array of int;
	   
	   Is the desired percentage of free space left in the database
	   \file after the database has been shrunk                      */
	bool __fastcall ShrinkDBs(TTntStringList* DBNames, int* Percents);
	/* Backs up an entire database, transaction log, or one or more
	   files or filegroups
	   
	   BackupInfo: Pointer to a TDBBackupInfo class
	   
	   This function uses DBName, DBFileList, DeviceName, DiskFile
	   \members of the TDBBackupInfo class                          */
	bool __fastcall BackupDBFiles(TDBBackupInfo* BackupInfo);
	/* Add a backup device.
	   
	   Type: Tape/Disk/Pipe
	   
	   DevName: Device name
	   
	   Path: Path to device */
	bool __fastcall AddBackupDevice(AnsiString Type, WideString DevName, WideString Path);
	/* Drops a database device or backup device from Microsoft SQL
	   Server, deleting the entry from master.dbo.sysdevices.
	   
	   DevName: Backup device name
	   
	   DeleteFile: Is whether or not the physical backup device file
	   should be deleted. delfile is varchar(7). If true, the
	   physical backup device disk file is deleted                   */
	bool __fastcall DeleteBackupDevice(WideString DevName, bool DeleteFile);
	/* Check if backup device exists
	   
	   DevName: Device name to check */
	bool __fastcall IsDeviceExist(WideString DevName);
	/* Get all backup devices
	   
	   After return, DeviceList will contain XML data in the
	   following format
	     * <c>\<BackupDevices\></c>
	       * <c>\<Device\></c>
	         * <c>\<Name\>\</Name\></c>
	         * <c>\<Size\>\</Size\></c>
	         * <c>\<Status\>\</Status\></c>
	         * <c>\<ControlType\>\</ControlType\></c>
	         * <c>\<PhysicalName\>\</PhysicalName\></c>
	       * <c>\</Device\></c>
	     * <c>\</BackupDevices\></c>                         */
	bool __fastcall GetBackupDevices(TTntStringList* DeviceList);
	/* Truncate the transaction log */
	bool __fastcall TruncateDBLogs(TTntStringList* DBNames);
	/* \Returns a result set with a list of the database and log
	   files contained in the backup set
	   
	   Files: File list
	   
	   DevName: Device name
	   
	   DiskFile: Device is a disk file
	   
	   BackupSet: Backup set number
	   
	   After return, Files will contain XML data in the following
	   format:
	     * \<BackupFiles\>
	       * \<BackupFile\>
	         * \<Name\>\</Name\>
	         * \<Path\>\</Path\>
	         * \<Type\>\</Type\>
	         * \<FileGroup\>\</FileGroup\>
	         * \<Size\>\</Size\>
	         * \<MaxSize\>\</MaxSize\>
	       * \</BackupFile\>
	     * \</BackupFiles\>                                       */
	bool __fastcall GetBackupFiles(TTntStringList* Files,WideString DevName, bool DiskFile, int BackupSet);
	/* Restore database files
	   
	   DBName
	   
	   DBFileList
	   
	   DeviceName
	   
	   DiskFile
	   
	   BackupSetNumber
	   
	   NewPathList            */
	bool __fastcall RestoreDBFiles(TDBBackupInfo* BackupInfo);
	/* Restore database log
	   
	   DBName
	   
	   DeviceName
	   
	   DiskFile
	   
	   BackupSetNumber
	   
	   LogFileName
	   
	   NewLogPath           */
	bool __fastcall RestoreDBLog(TDBBackupInfo* BackupInfo);
	/* Creates an entry in the security system that allows a user in
	   the current database to work with data in the current
	   database
	   
	   PermissionInfos: Pointer to array of TPermissionInfo
	   
	   Count: Number of array elements
	   
	   Permissions="INSERT, DELETE...                                */
	bool __fastcall SetPermissions(TPermissionInfo* PermissionInfos, int Count);
	/* Change object's owner */
	bool __fastcall ChangeObjectOwner(WideString Object, WideString NewOwner);
	/* Report information about user permissions for an object, in
	   the current database.
	   
	   Object: Object name
	   
	   User: Database user name
	   
	   Permissions: Permission list
	   
	   After return, Permissions will contain XML data in the
	   following format:
	     * \<Permissions\>
	       * \<Permission\>
	         * \<Owner\>\</Owner\>
	         * \<Object\>\</Object\>
	         * \<Grantee\>\</Grantee\>
	         * \<Grantor\>\</Grantor\>
	         * \<Type\>\</Type\>
	         * \<Action\>\</Action\>
	         * \<Column\>\</Column\>
	       * \</Permission\>
	     * \<Permissions\>                                         */
	bool __fastcall GetPermissions(WideString Object, WideString User,TTntStringList* Permissions);
/* Allow or Deny a Microsoft Windows NT user or group account
   to connect to Microsoft SQL Server using Windows
   Authentication.
   
   AccountInfos: Pointer to Array of TLoginInfo
   
   Count: Number of array elements
   
   This function uses LoginName, Option, Role members of
   TLoginInfo structure
   
   LoginName: Windows User or group
   
   Option: if true, deny access.
   
   Role: If not empty, add login to Server role                 */
	bool __fastcall GrantWindowLogins(TLoginInfo* AccountInfos, int Count);
	/* Removes the login entries from Microsoft SQL Server for a
	   Microsoft Windows NT user or group                         */
	bool __fastcall RemoveWindowLogins(TLoginInfo* AccountInfos, int Count);
	bool __fastcall GetServerRoles(TTntStringList* ServerRoles);
	/* \Returns information about the members of a Microsoft SQL
	   Server fixed server role.                                 */
	bool __fastcall GetServerRoleMembers(WideString ServerRole,TTntStringList* Members);
	/* Add/Remove login as member of fixed server roles
	   
	   AccountInfos: Pointer to array of TLoginInfo
	   
	   Role: Database role
	   
	   Option: If true, remove user from database role
	   
	   UserName: Database user name                     */
	bool __fastcall SetServerRoleMembers(TLoginInfo* AccountInfos, int Count);
	bool __fastcall GetDBRoles(TTntStringList* DBRoles);
	/* \Returns information about the members of a role in the
	   current database.
	   
	   DBRole: Database role name
	   
	   \Members: Member list
	   
	   After return, Members will contain XML data in the following
	   format:
	     * \<Members\>
	       * \<Name\>\</Name\>
	     * \</Members\>                                             */
	bool __fastcall GetDBRoleMembers(WideString DBRole, TTntStringList* Members);
	/* Add/Remove security accounts as members of existing
	   Microsoft SQL Server database roles in the current
	   database.
	   
	   AccountInfos: Pointer to array of TLoginInfo
	   
	   Role: Database role
	   
	   Option: If true, remove user from database role
	   
	   UserName: Database user name                         */
	bool __fastcall SetDBRoleMembers(TLoginInfo* AccountInfos, int Count);
	/* Creates a new Microsoft SQL Server role in the current
	   database.
	   
	   AccountInfos: Pointer to array of TLoginInfo
	   
	   Count: Number of array elements
	   
	   This function uses Role, LoginName members of TLoginInfo
	   
	   Role: Database role to create
	   
	   LoginName: New role owner.                               */
	bool __fastcall CreateDBRoles(TLoginInfo* AccountInfos, int Count);
	/* Removes Microsoft SQL Server roles from the current
	   database.
	   
	   DBRoles: Database role list                           */
	bool __fastcall DeleteDBRoles(TTntStringList* DBRoles);
	bool __fastcall CreateAppRoles(TLoginInfo* AppRoleInfos, int Count);
	/* Removes application roles from the current database.
	   
	   AppRoles: Application role list                      */
	bool __fastcall DeleteAppRoles(TTntStringList* AppRoles);
	/* Activates the permissions associated with an application role
	   in the current database.
	   
	   AppRoleInfo: Pointer to TLoginInfo
	   
	   Role: Application role
	   
	   Password: Password
	   
	   Option: If true, encrypt password before sending to Server    */
	bool __fastcall SetAppRole(TLoginInfo *AppRoleInfo);
	/* Changes the password of an application role in the current
	   database.
	   
	   AppRoleInfos: Array of TLoginInfo
	   
	   Count: Number of elements
	   
	   This function uses Role and Password member of TLoginInfo  */
	bool __fastcall ChangeAppRolePassword(TLoginInfo *AppRoleInfos, int Count);
__published:
	/* SQL server name */
	__property WideString ServerName= {read=ReadServerName};
	/* Current database name */
	__property WideString DatabaseName= {read=ReadDatabaseName, write=SetDatabase, stored=false};
	/* Login name or Windows account name */
	__property WideString LoginName= {read=ReadLoginName};
	/* Database user name */
	__property WideString DBUser= {read=ReadUserName};
	/* PID number */
	__property AnsiString ConnectionID= {read=ReadID};
	/* Display error message */
	__property bool DisplayError= {read=FDisplayError, write=FDisplayError};
	/* Windows computer name */
	__property AnsiString ComputerName= {read=FComputerName};
	/* Window user name */
	__property AnsiString WindowsUserName= {read=FUserName};
	/* Last error message */
	__property AnsiString LastError= {read=FLastError};
};
//---------------------------------------------------------------------------
#endif
